@@ -4,6 +4,9 @@ using Mono.Unix;
 using NDesk.DBus;
 using org.freedesktop.DBus;
 
+using FSpot.UI.Dialog;
+using FSpot.Utils;
+
 namespace FSpot {
 	[Interface ("org.gnome.FSpot.Core")]
 	public interface ICore {
@@ -14,6 +17,8 @@ namespace FSpot {
 		void View (string list);
 
 		void Shutdown ();
+
+		string Version ();
 	}
 
 	public class Core : ICore
@@ -32,19 +37,26 @@ namespace FSpot {
 		{
 			toplevels = new System.Collections.ArrayList ();
 			
-			if (!nodb) {
-				// Load the database, upgrading/creating it as needed
-				string base_directory = FSpot.Global.BaseDirectory;
-				if (! File.Exists (base_directory))
-					Directory.CreateDirectory (base_directory);
-				
-				db = new Db ();
-				db.Init (Path.Combine (base_directory, "photos.db"), true);
-			}
 		}
 
 		public static Db Database {
-			get { return db; }
+			get { 
+				if (db == null) {
+					// Load the database, upgrading/creating it as needed
+					string base_directory = FSpot.Global.BaseDirectory;
+					if (! File.Exists (base_directory))
+						Directory.CreateDirectory (base_directory);
+					
+					db = new Db ();
+					try {
+						db.Init (Path.Combine (base_directory, "photos.db"), true);
+					} catch (System.Exception e) {
+						new RepairDbDialog (e, db.Repair (), null);
+						db.Init (Path.Combine (base_directory, "photos.db"), true);
+					}
+				}
+				return db; 
+			}
 		}
 
 		public static ICore FindInstance ()
@@ -63,9 +75,11 @@ namespace FSpot {
 
 		public void RegisterServer ()
 		{
-			RequestNameReply nameReply = Bus.Session.RequestName (ServicePath, NameFlag.DoNotQueue);
 #if DEBUG_DBUS
+			RequestNameReply nameReply = Bus.Session.RequestName (ServicePath, NameFlag.DoNotQueue);
 			Console.WriteLine("RequestNameReply {0}", nameReply);
+#else
+			Bus.Session.RequestName (ServicePath, NameFlag.DoNotQueue);
 #endif
 			Bus.Session.Register (ServicePath, CorePath, this);
 		}
@@ -101,7 +115,7 @@ namespace FSpot {
 		public MainWindow MainWindow {
 			get {
 				if (organizer == null) {
-					organizer = new MainWindow (db);
+					organizer = new MainWindow (Database);
 					Register (organizer.Window);
 				}
 				
@@ -143,15 +157,17 @@ namespace FSpot {
 				Tag tag;
 				
 				if (name != null)
-					tag = db.Tags.GetTagByName (name);
+					tag = Database.Tags.GetTagByName (name);
 				else {
-					int id = (int) Preferences.Get (Preferences.SCREENSAVER_TAG);
-					tag = db.Tags.GetTagById (id);
+					int id = Preferences.Get<int> (Preferences.SCREENSAVER_TAG);
+					tag = Database.Tags.GetTagById (id);
 				}
 				
 				Photo [] photos;
 				if (tag != null)
-					photos = db.Photos.Query (new Tag [] { tag } );
+					photos = Database.Photos.Query (new Tag [] { tag } );
+ 				else if (Preferences.Get<int> (Preferences.SCREENSAVER_TAG) == 0)
+ 					photos = db.Photos.Query (new Tag [] {});
 				else
 					photos = new Photo [0];
 
@@ -169,13 +185,13 @@ namespace FSpot {
 					Gtk.HBox hbox = new Gtk.HBox ();
 					Gtk.VBox vbox = new Gtk.VBox ();
 
-					outer.PackStart (new Gtk.Label (""));
+					outer.PackStart (new Gtk.Label (String.Empty));
 					outer.PackStart (vbox, false, false, 0);
-					vbox.PackStart (new Gtk.Label (""));
+					vbox.PackStart (new Gtk.Label (String.Empty));
 					vbox.PackStart (hbox, false, false, 0);
 					hbox.PackStart (new Gtk.Image (Gtk.Stock.DialogWarning, Gtk.IconSize.Dialog),
 							false, false, 0);
-					outer.PackStart (new Gtk.Label (""));
+					outer.PackStart (new Gtk.Label (String.Empty));
 
 					string msg;
 					string long_msg;
@@ -187,8 +203,8 @@ namespace FSpot {
 											     "F-Spot preference dialog."), tag.Name);
 					} else {
 						msg = Catalog.GetString ("Search returned no results");
-						long_msg = Catalog.GetString ("The tag F-Spot is looking for does not exist. Try\n" +
-									      "selecting a different tag in the F-Spot preference\n" +
+						long_msg = Catalog.GetString ("The tag F-Spot is looking for does not exist. Try\n" + 
+									      "selecting a different tag in the F-Spot preference\n" + 
 									      "dialog.");
 					}
 
@@ -199,7 +215,7 @@ namespace FSpot {
 					long_label.Markup  = String.Format ("<small>{0}</small>", long_msg);
 
 					vbox.PackStart (long_label, false, false, 0);
-					vbox.PackStart (new Gtk.Label (""));
+					vbox.PackStart (new Gtk.Label (String.Empty));
 
 					window.Add (outer);
 					SetStyle (label);
@@ -246,13 +262,19 @@ namespace FSpot {
 			window.Destroyed += HandleDestroyed;
 		}
 
+		public string Version () {
+			return FSpot.Defines.VERSION; 
+		}
+
 		public void HandleDestroyed (object sender, System.EventArgs args)
 		{
+			Log.Information ("Exiting");
 			toplevels.Remove (sender);
 			if (toplevels.Count == 0) {
-				// FIXME
-				// Should use Application.Quit(), but for that to work we need to terminate the threads
-				// first too.
+				Banshee.Kernel.Scheduler.Dispose ();
+				Core.Database.Dispose ();
+				PixbufLoader.Cleanup ();
+				Gtk.Application.Quit ();
 				System.Environment.Exit (0);
 			}
 			if (organizer != null && organizer.Window == sender)
