@@ -74,9 +74,9 @@ namespace FSpot
 			return string.Compare (photo1.Name, photo2.Name);
 		}
 	
-		public class CompareDateName : IComparer
+		public class CompareDateName : IComparer<IBrowsableItem>
 		{
-			public int Compare (object obj1, object obj2)
+			public int Compare (IBrowsableItem obj1, IBrowsableItem obj2)
 			{
 				Photo p1 = (Photo)obj1;
 				Photo p2 = (Photo)obj2;
@@ -116,11 +116,26 @@ namespace FSpot
 			}
 		}
 	
+		PhotoChanges changes = new PhotoChanges ();
+		public PhotoChanges Changes {
+			get{ return changes; }
+			set {
+				if (value != null)
+					throw new ArgumentException ("The only valid value is null");
+				changes = new PhotoChanges ();
+			}
+		}
+
 		// The time is always in UTC.
 		private DateTime time;
 		public DateTime Time {
 			get { return time; }
-			set { time = value; }
+			set {
+				if (time == value)
+					return;
+				time = value;
+				changes.TimeChanged = true;
+			}
 		}
 	
 		public string Name {
@@ -151,29 +166,59 @@ namespace FSpot
 		private bool loaded = false;
 		public bool Loaded {
 			get { return loaded; }
-			set { loaded = value; }
+			set { 
+				if (value) {
+					if (DefaultVersionId != OriginalVersionId && !Versions.ContainsKey (DefaultVersionId)) 
+						DefaultVersionId = OriginalVersionId;	
+				}
+				loaded = value; 
+			}
 		}
 	
 		private string description;
 		public string Description {
 			get { return description; }
-			set { description = value; }
+			set {
+				if (description == value)
+					return;
+				description = value;
+				changes.DescriptionChanged = true;
+			}
 		}
 	
 		private uint roll_id = 0;
 		public uint RollId {
 			get { return roll_id; }
-			set { roll_id = value; }
+			set {
+				if (roll_id == value)
+					return;
+				roll_id = value;
+				changes.RollIdChanged = true;
+			}
 		}
 	
 		private uint rating;
 		public uint Rating {
 			get { return rating; }
 			set {
-				if (value >= 0 && value <= 5)
-					rating = value;
+				if (rating == value || value < 0 || value > 5)
+					return;
+				rating = value;
+				changes.RatingChanged = true;
 			}
 		}
+
+		private string md5_sum;
+		public string MD5Sum {
+			get { return md5_sum; }
+			set { 
+				if (md5_sum == value)
+				 	return;
+
+				md5_sum = value; 
+				changes.MD5SumChanged = true;
+			} 
+		}
 	
 		// Version management
 		public const int OriginalVersionId = 1;
@@ -211,16 +256,22 @@ namespace FSpot
 		private uint default_version_id = OriginalVersionId;
 		public uint DefaultVersionId {
 			get { return default_version_id; }
-			set { default_version_id = value; }
+			set {
+				if (default_version_id == value)
+					return;
+				default_version_id = value;
+				changes.DefaultVersionIdChanged = true;
+			}
 		}
 	
 		// This doesn't check if a version of that name already exists, 
 		// it's supposed to be used only within the Photo and PhotoStore classes.
-		internal void AddVersionUnsafely (uint version_id, System.Uri uri, string name, bool is_protected)
+		internal void AddVersionUnsafely (uint version_id, System.Uri uri, string md5_sum, string name, bool is_protected)
 		{
-			Versions [version_id] = new PhotoVersion (this, version_id, uri, name, is_protected);
+			Versions [version_id] = new PhotoVersion (this, version_id, uri, md5_sum, name, is_protected);
 	
 			highest_version_id = Math.Max (version_id, highest_version_id);
+			changes.AddVersion (version_id);
 		}
 	
 		public uint AddVersion (System.Uri uri, string name)
@@ -233,7 +284,11 @@ namespace FSpot
 			if (VersionNameExists (name))
 				throw new ApplicationException ("A version with that name already exists");
 			highest_version_id ++;
-			Versions [highest_version_id] = new PhotoVersion (this, highest_version_id, uri, name, is_protected);
+			string md5_sum = GenerateMD5 (uri);
+
+			Versions [highest_version_id] = new PhotoVersion (this, highest_version_id, uri, md5_sum, name, is_protected);
+
+			changes.AddVersion (highest_version_id);
 			return highest_version_id;
 		}
 	
@@ -350,7 +405,9 @@ namespace FSpot
 				PhotoStore.DeleteThumbnail (uri);
 			}
 			Versions.Remove (version_id);
-	
+
+			changes.RemoveVersion (version_id);
+
 			do {
 				version_id --;
 				if (Versions.ContainsKey (version_id)) {
@@ -373,6 +430,7 @@ namespace FSpot
 		{
 			System.Uri new_uri = GetUriForVersionName (name, System.IO.Path.GetExtension (VersionUri (base_version_id).AbsolutePath));
 			System.Uri original_uri = VersionUri (base_version_id);
+			string md5_sum = MD5Sum;
 	
 			if (VersionNameExists (name))
 				throw new Exception ("This version name already exists");
@@ -397,9 +455,15 @@ namespace FSpot
 	//				try {
 	//					Mono.Unix.Native.Syscall.chown(new_path, Mono.Unix.Native.Syscall.getuid (), stat.st_gid);
 	//				} catch (Exception) {}
+	//
+			} else {
+				md5_sum = Photo.GenerateMD5 (new_uri);
 			}
 			highest_version_id ++;
-			Versions [highest_version_id] = new PhotoVersion (this, highest_version_id, new_uri, name, is_protected);
+
+			Versions [highest_version_id] = new PhotoVersion (this, highest_version_id, new_uri, md5_sum, name, is_protected);
+
+			changes.AddVersion (highest_version_id);
 	
 			return highest_version_id;
 		}
@@ -421,8 +485,10 @@ namespace FSpot
 					continue;
 	
 				highest_version_id ++;
-				Versions [highest_version_id] = new PhotoVersion (this, highest_version_id, version.Uri, name, is_protected);
-	
+				Versions [highest_version_id] = new PhotoVersion (this, highest_version_id, version.Uri, version.MD5Sum, name, is_protected);
+
+				changes.AddVersion (highest_version_id);
+
 				return highest_version_id;
 			}
 		}
@@ -473,6 +539,8 @@ namespace FSpot
 				throw new Exception ("This name already exists");
 	
 			(GetVersion (version_id) as PhotoVersion).Name = new_name;
+
+			changes.ChangeVersion (version_id);
 	
 			//TODO: rename file too ???
 	
@@ -493,6 +561,7 @@ namespace FSpot
 				tags = new ArrayList ();
 	
 			tags.Add (tag);
+			changes.AddTag (tag);
 		}
 	
 		// This on the other hand does, but is O(n) with n being the number of existing tags.
@@ -514,8 +583,11 @@ namespace FSpot
 	
 		public void RemoveTag (Tag tag)
 		{
-			if (HasTag (tag))
-				tags.Remove (tag);
+			if (!HasTag (tag))
+				return;
+
+			tags.Remove (tag);
+			changes.RemoveTag (tag);
 		}
 	
 		public void RemoveTag (Tag []taglist)
@@ -544,9 +616,52 @@ namespace FSpot
 			return tags.Contains (tag);
 		}
 	
+		//
+		// MD5 Calculator
+		//
+		private static System.Security.Cryptography.MD5 md5_generator;
+
+		private static System.Security.Cryptography.MD5 MD5Generator {
+			get {
+				if (md5_generator == null)
+				 	md5_generator = new System.Security.Cryptography.MD5CryptoServiceProvider ();
+
+				return md5_generator;
+			} 
+		}
+
+		private static IDictionary<System.Uri, string> md5_cache = new Dictionary<System.Uri, string> ();
+
+		public static void ResetMD5Cache () {
+			if (md5_cache != null)	
+				md5_cache.Clear (); 
+		}
+
+		public static string GenerateMD5 (System.Uri uri)
+		{
+		 	try {
+			 	if (md5_cache.ContainsKey (uri))
+				 	return md5_cache [uri];
+
+				using (Gdk.Pixbuf pixbuf = ThumbnailGenerator.Create (uri))
+				{
+					byte[] serialized = PixbufSerializer.Serialize (pixbuf);
+					byte[] md5 = MD5Generator.ComputeHash (serialized);
+					string md5_string = Convert.ToBase64String (md5);
+
+					md5_cache.Add (uri, md5_string);
+					return md5_string;
+				}
+			} catch (Exception e) {
+			 	Log.DebugFormat("Failed to create MD5Sum for Uri {0}; {1}", uri, e.Message);
+			}
+
+			return string.Empty; 
+		}
+
 
 		// Constructor
-		public Photo (uint id, long unix_time, System.Uri uri)
+		public Photo (uint id, long unix_time, System.Uri uri, string md5_sum)
 			: base (id)
 		{
 			if (uri == null)
@@ -556,10 +671,11 @@ namespace FSpot
 	
 			description = String.Empty;
 			rating = 0;
+			this.md5_sum = md5_sum;
 	
 			// Note that the original version is never stored in the photo_versions table in the
 			// database.
-			AddVersionUnsafely (OriginalVersionId, uri, Catalog.GetString ("Original"), true);
+			AddVersionUnsafely (OriginalVersionId, uri, md5_sum, Catalog.GetString ("Original"), true);
 		}
 	}
 }
