@@ -7,24 +7,6 @@ using System.Diagnostics;
 using FSpot;
 using FSpot.Utils;
 
-public class DbItemEventArgs {
-	private DbItem [] items;
-	
-	public DbItem [] Items {
-		get { return items; }
-	}
-	
-	public DbItemEventArgs (DbItem [] items)
-	{
-		this.items = items;
-	}
-	
-	public DbItemEventArgs (DbItem item)
-	{
-		this.items = new DbItem [] { item };
-	}
-}
-
 // A Store maps to a SQL table.  We have separate stores (i.e. SQL tables) for tags, photos and imports.
 
 public delegate void ItemsAddedHandler (object sender, DbItemEventArgs args);
@@ -163,6 +145,10 @@ public class Db : IDisposable {
 	bool empty;
 	string path;
 
+	public delegate void ExceptionThrownHandler (Exception e);
+	public event ExceptionThrownHandler ExceptionThrown;
+
+
 	public TagStore Tags {
 		get { return tag_store; }
 	}
@@ -231,6 +217,8 @@ public class Db : IDisposable {
 			throw new Exception (path + ": File not found");
 
 		database = new QueuedSqliteDatabase(path);
+		database.ExceptionThrown += HandleDbException;
+
 		if (database.GetFileVersion(path) == 2)
 			SqliteUpgrade ();
 
@@ -248,16 +236,18 @@ public class Db : IDisposable {
 		job_store = new JobStore (Database, new_db);
  		photo_store = new PhotoStore (Database, new_db);
 
-		FSpot.DBusProxyFactory.Load (this);
-		
 		Database.CommitTransaction ();
 
 		empty = new_db;
 		Log.DebugTimerPrint (timer, "Db Initialization took {0}");
 	}
 
-	public void EmitDown () {
-		FSpot.DBusProxyFactory.EmitRemoteDown ();
+	void HandleDbException (Exception e)
+	{
+		if (ExceptionThrown != null)
+			ExceptionThrown (e);
+		else
+			throw e;
 	}
 
 	public bool Empty {
@@ -268,7 +258,27 @@ public class Db : IDisposable {
 
 	public void Dispose ()
 	{
-		Database.Dispose ();
+		Dispose (true);
+		GC.SuppressFinalize (this);
+	}
+
+	bool already_disposed = false;
+	protected virtual void Dispose (bool is_disposing)
+	{
+		if (already_disposed)
+			return;
+		if (is_disposing) {//Free managed resources
+			Database.Dispose ();
+		}
+		//Free eunmanaged resources
+
+		already_disposed = true;
+	}
+
+	~Db ()
+	{
+		Log.DebugFormat ("Finalizer called on {0}. Should be Disposed", GetType ());
+		Dispose (false);
 	}
 
 	public void BeginTransaction()
