@@ -109,7 +109,7 @@ namespace FSpot {
 
 		public override FSpot.IBrowsableItem PhotoFromIndex (int item) 
 		{
-			return query.Items [LookupItem (item)];
+			return query [LookupItem (item)];
 		}
 
 		private int LookupItem (int group)
@@ -124,8 +124,8 @@ namespace FSpot {
 			return 0;
 		}
 
-		public DirectoryAdaptor (PhotoQuery query)
-			: base (query)
+		public DirectoryAdaptor (PhotoQuery query, bool order_ascending)
+			: base (query, order_ascending)
 		{ }
 	}
 }
