@@ -3,7 +3,7 @@ using Mono.Unix;
 using Gtk;
 using Gdk;
 using GLib;
-
+using FSpot.Utils;
 
 namespace FSpot {
 	public class GroupSelector : Fixed {
@@ -910,6 +910,14 @@ namespace FSpot {
 			if (event_window != null)
 				event_window.Show ();
 		}
+
+		protected override void OnUnmapped ()
+		{
+			base.OnUnmapped ();
+			if (event_window != null)
+				event_window.Hide ();
+			
+		}
 		
 		protected override bool OnExposeEvent (Gdk.EventExpose args)
 		{
@@ -923,9 +931,13 @@ namespace FSpot {
 					active.X = min_x;
 					active.Width = max_x - min_x;
 					
-					if (active.Intersect (area, out active)) {
+#if GTK_2_10_3 && !GTK_2_12_2 //workaround for buggy code
+					active.Intersect (area);
+					GdkWindow.DrawRectangle (Style.BaseGC (State), true, active);
+#else
+					if (active.Intersect (area, out active))
 						GdkWindow.DrawRectangle (Style.BaseGC (State), true, active);
-					}
+#endif
 					
 					int i;
 					BoxXHit (area.X, out i);
@@ -965,9 +977,10 @@ namespace FSpot {
 			
 		protected override void OnSizeRequested (ref Requisition requisition)
 		{
+			left.SizeRequest ();
+			right.SizeRequest ();
 			requisition.Width = 500;
 			requisition.Height = (int) (LegendHeight () + glass.handle_height + 3 * border);
-			base.OnSizeRequested (ref requisition);
 		}
 
 		// FIXME I can't find a c# wrapper for the C PANGO_PIXELS () macro
