@@ -8,22 +8,26 @@
  *
  */
 using Gtk;
+using FSpot.Widgets;
 
 namespace FSpot {
 	public class InfoItem : InfoBox {
 		BrowsablePointer item;
-
+		
 		public InfoItem (BrowsablePointer item)
 		{
 			this.item = item;
 			item.Changed += HandleItemChanged;
 			HandleItemChanged (item, null);
 			VersionIdChanged += HandleVersionIdChanged;
+			ShowTags = true;
+			ShowRating = true;
+			Context = ViewContext.FullScreen;
 		}
-
+		
 		private void HandleItemChanged (BrowsablePointer sender, BrowsablePointerChangedArgs args)
 		{
-			Photo = item.Current;
+			Photo = item.Current as Photo;
 		}
 
 		private void HandleVersionIdChanged (InfoBox box, uint version_id)
@@ -43,12 +47,13 @@ namespace FSpot {
 
 		public InfoOverlay (Widget w, BrowsablePointer item) : base (w)
 		{
-			XAlign = 0.9;
+			XAlign = 1.0;
 			YAlign = 0.1;
+			DefaultWidth = 250;
 			box = new InfoItem (item);
 			box.BorderWidth = 15;
 			Add (box);
-			box.Show ();
+			box.ShowAll ();
 			Visibility = VisibilityType.Partial;
 			KeepAbove = true;
 			//WindowPosition = WindowPosition.Mouse;
