@@ -1,19 +1,23 @@
 using Gtk;
 using Gdk;
 using System;
+using Mono.Addins;
 using Mono.Unix;
 
+using FSpot.Extensions;
 using FSpot.Utils;
 using FSpot.UI.Dialog;
+using FSpot.Widgets;
 
 namespace FSpot {
 	public class SingleView {
 		[Glade.Widget] Gtk.HBox toolbar_hbox;
 		[Glade.Widget] Gtk.VBox info_vbox;
 		[Glade.Widget] Gtk.ScrolledWindow image_scrolled;
-		[Glade.Widget] Gtk.ScrolledWindow directory_scrolled;
 		[Glade.Widget] Gtk.HPaned info_hpaned;
 
+		Gtk.ScrolledWindow directory_scrolled;
+
 		[Glade.Widget] Gtk.CheckMenuItem side_pane_item;
 		[Glade.Widget] Gtk.CheckMenuItem toolbar_item;
 		[Glade.Widget] Gtk.CheckMenuItem filenames_item;
@@ -27,14 +31,14 @@ namespace FSpot {
 
 		[Glade.Widget] Label status_label;
 
+		Sidebar sidebar;
+
 		protected Glade.XML xml;
 		private Gtk.Window window;
 		PhotoImageView image_view;
 		FSpot.Widgets.IconView directory_view;
 		private Uri uri;
 		
-		InfoDialog metadata_dialog;
-		
 		UriCollection collection;
 		
 		FullScreenView fsview;
@@ -108,8 +112,22 @@ namespace FSpot {
 			directory_view.DisplayTags = false;
 			directory_view.DisplayDates = false;
 			directory_view.DisplayRatings = false;
+
+			directory_scrolled = new ScrolledWindow();
 			directory_scrolled.Add (directory_view);
 
+			sidebar = new Sidebar ();
+
+			info_vbox.Add (sidebar);
+			sidebar.AppendPage (directory_scrolled, Catalog.GetString ("Folder"), "gtk-directory");
+
+			AddinManager.AddExtensionNodeHandler ("/FSpot/Sidebar", OnSidebarExtensionChanged);
+ 		
+			sidebar.Context = ViewContext.Single;
+
+			sidebar.CloseRequested += HandleHideSidePane;
+			sidebar.Show ();
+
 			ThumbnailGenerator.Default.OnPixbufLoaded += delegate { directory_view.QueueDraw (); };
 
 			image_view = new PhotoImageView (collection);
@@ -140,16 +158,32 @@ namespace FSpot {
 			window.DeleteEvent += HandleDeleteEvent;
 			
 			collection.Changed += HandleCollectionChanged;
+
+			// wrap the methods to fit to the delegate
+			image_view.Item.Changed += delegate (BrowsablePointer pointer, BrowsablePointerChangedArgs old) {
+															IBrowsableItem [] item = {pointer.Current};
+															PhotoArray item_array = new PhotoArray (item);
+															sidebar.HandleSelectionChanged (item_array);
+													};
+			
+			image_view.Item.Collection.ItemsChanged += sidebar.HandleSelectionItemsChanged;
+
 			UpdateStatusLabel ();
 			
 			if (collection.Count > 0)
 				directory_view.Selection.Add (0);
 
-			export.Submenu = (Mono.Addins.AddinManager.GetExtensionNode ("/FSpot/Menus/Exports") as FSpot.Extensions.SubmenuNode).GetMenuItem ().Submenu;
+			export.Submenu = (Mono.Addins.AddinManager.GetExtensionNode ("/FSpot/Menus/Exports") as FSpot.Extensions.SubmenuNode).GetMenuItem (this).Submenu;
 			export.Submenu.ShowAll ();
 			export.Activated += HandleExportActivated ;
 		}
 
+		private void OnSidebarExtensionChanged (object s, ExtensionNodeEventArgs args) {
+			// FIXME: No sidebar page removal yet!
+			if (args.Change == ExtensionChange.Add)
+				sidebar.AppendPage ((args.ExtensionNode as SidebarPageNode).GetSidebarPage ());
+		}
+
 		void HandleExportActivated (object o, EventArgs e)
 		{
 			FSpot.Extensions.ExportMenuItemNode.SelectedImages = delegate () {return new FSpot.PhotoArray (directory_view.Selection.Items); };
@@ -205,17 +239,15 @@ namespace FSpot {
 		void HandleRotate90Command (object sender, System.EventArgs args) 
 		{
 			RotateCommand command = new RotateCommand (this.Window);
-			if (command.Execute (RotateDirection.Clockwise, new IBrowsableItem [] { image_view.Item.Current })) {
-				collection.MarkChanged (image_view.Item.Index);
-			}
+			if (command.Execute (RotateDirection.Clockwise, new IBrowsableItem [] { image_view.Item.Current }))
+				collection.MarkChanged (image_view.Item.Index, FullInvalidate.Instance);
 		}
 
 		void HandleRotate270Command (object sender, System.EventArgs args) 
 		{
 			RotateCommand command = new RotateCommand (this.Window);
-			if (command.Execute (RotateDirection.Counterclockwise, new IBrowsableItem [] { image_view.Item.Current })) {
-				collection.MarkChanged (image_view.Item.Index);
-			}
+			if (command.Execute (RotateDirection.Counterclockwise, new IBrowsableItem [] { image_view.Item.Current }))
+				collection.MarkChanged (image_view.Item.Index, FullInvalidate.Instance);
 		}		
 
 		private void HandleSelectionChanged (FSpot.IBrowsableCollection selection) 
@@ -225,8 +257,6 @@ namespace FSpot {
 				image_view.Item.Index = ((FSpot.Widgets.IconView.SelectionCollection)selection).Ids[0];
 
 				zoom_scale.Value = image_view.NormalizedZoom;
-				if (metadata_dialog != null)
-					metadata_dialog.InfoDisplay.Photo = image_view.Item.Current;
 			}
 			UpdateStatusLabel ();
 		}
@@ -274,20 +304,6 @@ namespace FSpot {
 			fsview.PlayPause ();
 		}
 	
-		private void HandleViewMetadata (object sender, System.EventArgs args)
-		{
-			if (metadata_dialog != null) {
-				metadata_dialog.Present ();
-				return;
-			}
-			
-			metadata_dialog = new InfoDialog (window);
-			metadata_dialog.InfoDisplay.Photo = image_view.Item.Current;
-			
-			metadata_dialog.ShowAll ();
-			metadata_dialog.Destroyed += HandleMetadataDestroyed;
-		}
-
 		private void HandleViewFilenames (object sender, System.EventArgs args)
 		{
 			directory_view.DisplayFilenames = filenames_item.Active; 
@@ -344,11 +360,6 @@ namespace FSpot {
 			chooser.Destroy ();
 		}
 
-		private void HandleMetadataDestroyed (object sender, System.EventArgs args)
-		{
-			metadata_dialog = null;
-		}
-	
 		private bool SlideShow ()
 		{
 			IBrowsableItem [] items = new IBrowsableItem [collection.Count];
@@ -520,14 +531,9 @@ namespace FSpot {
 
 		void LoadPreference (String key)
 		{
-			object val = Preferences.Get (key);
-
-			if (val == null)
-				return;
-			
 			switch (key) {
 			case Preferences.VIEWER_MAXIMIZED:
-				if ((bool) val)
+				if (Preferences.Get<bool> (key))
 					window.Maximize ();
 				else
 					window.Unmaximize ();
@@ -535,43 +541,43 @@ namespace FSpot {
 
 			case Preferences.VIEWER_WIDTH:
 			case Preferences.VIEWER_HEIGHT:
-				window.SetDefaultSize((int) Preferences.Get(Preferences.VIEWER_WIDTH),
-						(int) Preferences.Get(Preferences.VIEWER_HEIGHT));
+				window.SetDefaultSize(Preferences.Get<int> (Preferences.VIEWER_WIDTH),
+						      Preferences.Get<int> (Preferences.VIEWER_HEIGHT));
 
 				window.ReshowWithInitialSize();
 				break;
 			
 			case Preferences.VIEWER_SHOW_TOOLBAR:
-				if (toolbar_item.Active != (bool) val)
-					toolbar_item.Active = (bool) val;
+				if (toolbar_item.Active != Preferences.Get<bool> (key))
+					toolbar_item.Active = Preferences.Get<bool> (key);
 
-				toolbar_hbox.Visible = (bool) val;
+				toolbar_hbox.Visible = Preferences.Get<bool> (key);
 				break;
 
 			case Preferences.VIEWER_INTERPOLATION:
-				if ((bool) val)
+				if (Preferences.Get<bool> (key))
 					image_view.Interpolation = Gdk.InterpType.Bilinear;
 				else
 					image_view.Interpolation = Gdk.InterpType.Nearest;
 				break;
 
 			case Preferences.VIEWER_SHOW_FILENAMES:
-				if (filenames_item.Active != (bool) val)
-					filenames_item.Active = (bool) val;
+				if (filenames_item.Active != Preferences.Get<bool> (key))
+					filenames_item.Active = Preferences.Get<bool> (key);
 				break;
 
 			case Preferences.VIEWER_TRANSPARENCY:
-				if (val as string == "CHECK_PATTERN")
+				if (Preferences.Get<string> (key) == "CHECK_PATTERN")
 					image_view.SetCheckSize (2);
-				else if (val as string == "COLOR")
-					image_view.SetTransparentColor (Preferences.Get(Preferences.VIEWER_TRANS_COLOR) as string);
+				else if (Preferences.Get<string> (key) == "COLOR")
+					image_view.SetTransparentColor (Preferences.Get<string> (Preferences.VIEWER_TRANS_COLOR));
 				else // NONE
 					image_view.SetTransparentColor (image_view.Style.BaseColors [(int)Gtk.StateType.Normal]);
 				break;
 
 			case Preferences.VIEWER_TRANS_COLOR:
-				if (Preferences.Get(Preferences.VIEWER_TRANSPARENCY) as string == "COLOR")
-					image_view.SetTransparentColor (val as string);
+				if (Preferences.Get<string> (Preferences.VIEWER_TRANSPARENCY) == "COLOR")
+					image_view.SetTransparentColor (Preferences.Get<string> (key));
 				break;
 			}
 		}
@@ -648,17 +654,13 @@ namespace FSpot {
 
 			void LoadPreference (string key)
 			{
-				object val = Preferences.Get (key);
-
-				if (val == null)
-					return;
-			
+		
 				switch (key) {
 				case Preferences.VIEWER_INTERPOLATION:
-					interpolation_check.Active = (bool) val;
+					interpolation_check.Active = Preferences.Get<bool> (key);
 					break;
 				case Preferences.VIEWER_TRANSPARENCY:
-					switch ((string) val) {
+					switch (Preferences.Get<string> (key)) {
 					case "COLOR":
 						as_color_radio.Active = true;
 						break;
@@ -672,9 +674,9 @@ namespace FSpot {
 					break;
 				case Preferences.VIEWER_TRANS_COLOR:
 					color_button.Color = new Gdk.Color (
-						Byte.Parse ((val as string).Substring (1,2), System.Globalization.NumberStyles.AllowHexSpecifier),
-						Byte.Parse ((val as string).Substring (3,2), System.Globalization.NumberStyles.AllowHexSpecifier),
-						Byte.Parse ((val as string).Substring (5,2), System.Globalization.NumberStyles.AllowHexSpecifier));
+						Byte.Parse (Preferences.Get<string> (key).Substring (1,2), System.Globalization.NumberStyles.AllowHexSpecifier),
+						Byte.Parse (Preferences.Get<string> (key).Substring (3,2), System.Globalization.NumberStyles.AllowHexSpecifier),
+						Byte.Parse (Preferences.Get<string> (key).Substring (5,2), System.Globalization.NumberStyles.AllowHexSpecifier));
 					break;
 				}
 			}
