@@ -192,8 +192,16 @@ public class TagSelectionWidget : FSpot.Widgets.SaneTreeView {
 		SetBackground (renderer, tag);
 
 		// FIXME I can't set the Pixbuf to null, not sure if it's a GTK# bug...
-		if (tag.Icon != null)
-			(renderer as CellRendererPixbuf).Pixbuf = tag.SizedIcon;
+		if (tag.Icon != null) {
+			if (FSpot.ColorManagement.IsEnabled) {
+				//FIXME
+				Gdk.Pixbuf temp = tag.SizedIcon.Copy();
+				FSpot.ColorManagement.ApplyScreenProfile (temp);
+				(renderer as CellRendererPixbuf).Pixbuf = temp;
+			}
+			else
+				(renderer as CellRendererPixbuf).Pixbuf = tag.SizedIcon;
+		}
 		else
 			(renderer as CellRendererPixbuf).Pixbuf = empty_pixbuf;
 	}
@@ -379,27 +387,23 @@ public class TagSelectionWidget : FSpot.Widgets.SaneTreeView {
 
 	void ExpandDefaults ()
 	{
-		object val = FSpot.Preferences.Get (FSpot.Preferences.EXPANDED_TAGS);
-
-		if (val == null)
+		int [] tags = FSpot.Preferences.Get<int []> (FSpot.Preferences.EXPANDED_TAGS);
+		if (tags == null) {
 			ExpandAll ();
-		else {
-			TreeIter [] iters = ModelIters ();
-			if (iters == null || iters.Length == 0)
-				return;
+			return;
+		}
 
-			ArrayList expanded_tags = new ArrayList (val as int[]);
-			if (expanded_tags.Count < 1)
-				return;
+		TreeIter [] iters = ModelIters ();
+		if (iters == null || iters.Length == 0 || tags.Length == 0)
+			return;
 
-			foreach (TreeIter iter in iters)
-			{
-				GLib.Value v = new GLib.Value ();
-				Model.GetValue (iter, IdColumn, ref v);
-				int tag_id = (int)(uint) v;
-				if (expanded_tags.Contains (tag_id)) {
-					ExpandRow (Model.GetPath (iter), false);
-				}
+		foreach (TreeIter iter in iters)
+		{
+			GLib.Value v = new GLib.Value ();
+			Model.GetValue (iter, IdColumn, ref v);
+			int tag_id = (int)(uint) v;
+			if (Array.IndexOf (tags, tag_id) > -1) {
+				ExpandRow (Model.GetPath (iter), false);
 			}
 		}
 	}
@@ -514,7 +518,7 @@ public class TagSelectionWidget : FSpot.Widgets.SaneTreeView {
 		}
 
 		tag.Name = args.NewText;
-		tag_store.Commit (tag);
+		tag_store.Commit (tag, true);
 
 		text_render.Edited -= HandleTagNameEdited;
 
