@@ -11,6 +11,7 @@ using Banshee.Database;
 using FSpot;
 using FSpot.Jobs;
 using FSpot.Query;
+using FSpot.Utils;
 
 // FIXME: This is to workaround the currently busted GTK# bindings.
 using System.Runtime.InteropServices;
@@ -34,7 +35,8 @@ public class PixbufSerializer {
 	public static byte [] Serialize (Pixbuf pixbuf)
 	{
 		Pixdata pixdata = new Pixdata ();
-		pixdata.FromPixbuf (pixbuf, true); // FIXME GTK# shouldn't this be a constructor or something?
+		IntPtr raw_pixdata = pixdata.FromPixbuf (pixbuf, false); // FIXME GTK# shouldn't this be a constructor or something?
+									//       It's probably because we need the IntPtr to free it afterwards
 
 		uint data_length;
 		IntPtr raw_data = gdk_pixdata_serialize (ref pixdata, out data_length);
@@ -42,8 +44,8 @@ public class PixbufSerializer {
 		byte [] data = new byte [data_length];
 		Marshal.Copy (raw_data, data, 0, (int) data_length);
 		
-		GLib.Marshaller.Free (raw_data);
-
+		GLib.Marshaller.Free (new IntPtr[] { raw_data, raw_pixdata });
+		
 		return data;
 	}
 }
@@ -156,7 +158,7 @@ public class TagStore : DbStore {
 
 		// Pass 1, get all the tags.
 
-		SqliteDataReader reader = Database.Query("SELECT id, name, is_category, sort_priority, icon FROM tags");
+		SqliteDataReader reader = Database.Query ("SELECT id, name, is_category, sort_priority, icon FROM tags");
 
 		while (reader.Read ()) {
 			uint id = Convert.ToUInt32 (reader [0]);
@@ -170,7 +172,11 @@ public class TagStore : DbStore {
 				tag = new Tag (null, id, name);
 
 			if (reader [4] != null)
-				SetIconFromString (tag, reader [4].ToString ());
+				try {
+					SetIconFromString (tag, reader [4].ToString ());
+				} catch (Exception ex) {
+					Log.Exception ("Unable to load icon for tag " + name, ex);
+				}
 
 			tag.SortPriority = Convert.ToInt32 (reader[3]);
 			AddToCache (tag);
@@ -201,8 +207,11 @@ public class TagStore : DbStore {
 
 		//Pass 3, set popularity
 		reader = Database.Query ("SELECT tag_id, COUNT (*) as popularity FROM photo_tags GROUP BY tag_id");
-		while (reader.Read ())
-			(Get (Convert.ToUInt32 (reader [0])) as Tag).Popularity = Convert.ToInt32 (reader [1]);
+		while (reader.Read ()) {
+			Tag t = Get (Convert.ToUInt32 (reader [0])) as Tag;
+			if (t != null)
+				t.Popularity = Convert.ToInt32 (reader [1]);
+		}
 		reader.Close ();
 
 		if (FSpot.Core.Database.Meta.HiddenTagId.Value != null)
@@ -365,6 +374,11 @@ public class TagStore : DbStore {
 	{
 		Tag tag = item as Tag;
 
+		bool use_transactions = !Database.InTransaction && update_xmp;
+
+		if (use_transactions)
+			Database.BeginTransaction ();
+
 		Database.ExecuteNonQuery (new DbCommand ("UPDATE tags SET name = :name, category_id = :category_id, "
                     + "is_category = :is_category, sort_priority = :sort_priority, icon = :icon WHERE id = :id",
 						  "name", tag.Name,
@@ -374,13 +388,16 @@ public class TagStore : DbStore {
 						  "icon", GetIconString (tag),
 						  "id", tag.Id));
 		
-		EmitChanged (tag);
-
-		if (update_xmp && (bool)Preferences.Get(Preferences.METADATA_EMBED_IN_IMAGE)) {
+		if (update_xmp && Preferences.Get<bool> (Preferences.METADATA_EMBED_IN_IMAGE)) {
 			Photo [] photos = Core.Database.Photos.Query (new Tag [] { tag });
-			foreach (Photo p in photos) {
-				SyncMetadataJob.Create (Core.Database.Jobs, p);
-			}
+			foreach (Photo p in photos)
+				if (p.HasTag (tag)) // the query returns all the pics of the tag and all its child. this avoids updating child tags
+					SyncMetadataJob.Create (Core.Database.Jobs, p);
 		}
+
+		if (use_transactions)
+			Database.CommitTransaction ();
+
+		EmitChanged (tag);
 	}
 }
