@@ -1,3 +1,12 @@
+/*
+ * FSpot.ThumbnailGenerator.cs
+ *
+ * Author(s)
+ * 	Larry Ewing  <lewing@novell.com>
+ *
+ * This is free software. See COPYING for details.
+ */
+
 using System;
 using System.IO;
 using FSpot.Utils;
@@ -57,6 +66,7 @@ namespace FSpot {
 			return large_path;
 		}
 
+		[Obsolete ("Use ThumbnailPath (System.Uri) instead")]
 		public static string ThumbnailPath (string path) 
 		{
 			return ThumbnailPath (UriUtils.PathToFileUri (path));
@@ -102,21 +112,25 @@ namespace FSpot {
 				
 		}
 
+		public override void Request (Uri uri, int order, int width, int height)
+		{
+			if (!uri.IsFile)
+				Log.Debug ("FIXME: compute timestamp on non file uri too");
+			if (uri.IsFile && System.IO.File.Exists (ThumbnailPath (uri)) 
+				&& System.IO.File.GetLastWriteTime (ThumbnailPath (uri)) >= System.IO.File.GetLastWriteTime (uri.AbsolutePath))
+				return;
+
+			base.Request (uri, order, width, height);
+		}
+
 		protected override void ProcessRequest (RequestItem request)
 		{
 			try {
 				base.ProcessRequest (request);
 
 				Gdk.Pixbuf image = request.result;
-				if (image != null) {
-					Uri uri;
-					if (File.Exists (request.path))
-						uri = UriUtils.PathToFileUri (request.path);
-					else
-						uri = new Uri (request.path);
-
-					Save (image, uri);
-				}
+				if (image != null)
+					Save (image, request.uri);
 
 				System.Threading.Thread.Sleep (75);
 			} catch (System.Exception e) {
