@@ -1,56 +1,34 @@
+/*
+ * FSpot.TimeAdaptor.cs
+ *
+ * Author(s):
+ *	Larry Ewing  <lewing@novell.com>
+ * 	Stephane Delcroix  <stephnae@delcroix.org>
+ *
+ * This is free software. See COPYING for details.
+ */
+
 using System;
-using System.Collections;
+using System.Threading;
+using System.Collections.Generic;
 using FSpot.Query;
 using FSpot.Utils;
 
 namespace FSpot {
 	public class TimeAdaptor : GroupAdaptor, FSpot.ILimitable {
-		ArrayList years = new ArrayList ();
-		struct YearData {
-			public int Year;
-			public int [] Months;
-		}
+		Dictionary <int, int[]> years = new Dictionary<int, int[]> ();
 
 		public override event GlassSetHandler GlassSet;
 		public override void SetGlass (int min)
 		{
 			DateTime date = DateFromIndex (min);
-			
+
 			if (GlassSet != null)
-				GlassSet (this, LookupItem (date));
+				GlassSet (this, query.LookupItem (date));
 		}
 
-		public int LookupItem (System.DateTime date)
-		{
-			if (order_ascending) 
-				return LookUpItemAscending (date);
-			
-			return LookUpItemDescending (date);
-		}
-
-		private int LookUpItemAscending (System.DateTime date)
-		{
-			int i = 0;
-
-			while (i < query.Count && query [i].Time < date)
-				i++;
-
-			return i;
-		}
-
-		private int LookUpItemDescending (System.DateTime date)
-		{
-			int i = 0;
-
-			while (i < query.Count && query [i].Time > date)
-				i++;
-
-			return i;
-		}
-		
 		public void SetLimits (int min, int max) 
 		{
-			//Console.WriteLine ("min {0} max {1}", min, max);
 			DateTime start = DateFromIndex (min);
 
 			DateTime end = DateFromIndex(max);
@@ -65,11 +43,7 @@ namespace FSpot {
 
 		public void SetLimits (DateTime start, DateTime end)
 		{
-			//Console.WriteLine ("{0} {1}", start, end);
-			if (start > end)
-				query.Range = new DateRange (end, start);
-			else 
-				query.Range = new DateRange (start, end);
+			query.Range = (start > end) ? new DateRange (end, start) : new DateRange (start, end);
 		}
 
 		public override int Count ()
@@ -79,9 +53,7 @@ namespace FSpot {
 
 		public override string GlassLabel (int item)
 		{
-			DateTime start = DateFromIndex (item);
-			
-			return String.Format ("{0} ({1})", start.ToString ("MMMM yyyy"), Value (item));
+			return String.Format ("{0} ({1})", DateFromIndex (item).ToString ("MMMM yyyy"), Value (item));
 		}
 
 		public override string TickLabel (int item)
@@ -96,9 +68,10 @@ namespace FSpot {
 
 		public override int Value (int item)
 		{
-			YearData data = (YearData)years [item/12];
-
-			return data.Months [item % 12];
+			if (order_ascending)
+				return years [startyear + item/12][item % 12];
+			else
+				return years [endyear - item/12][11 - item % 12];
 		}
 
 		public DateTime DateFromIndex (int item) 
@@ -114,7 +87,7 @@ namespace FSpot {
 
 		private DateTime DateFromIndexAscending (int item)
 		{
-			int year = (int)((YearData)years [item / 12]).Year;
+			int year = startyear + item/12;
 			int month = 1 + (item % 12);
 
 			return new DateTime(year, month, 1);
@@ -122,7 +95,7 @@ namespace FSpot {
 
 		private DateTime DateFromIndexDescending (int item)
 		{
-			int year =  (int)((YearData)years [item / 12]).Year;
+			int year = endyear - item/12;
 			int month = 12 - (item % 12);
 			
 			return new DateTime (year, month, DateTime.DaysInMonth (year, month)).AddDays (1.0).AddMilliseconds (-.1);
@@ -147,95 +120,66 @@ namespace FSpot {
 		private int IndexFromDateAscending(DateTime date)
 		{
 			int year = date.Year;
-			int max_year = ((YearData)years [years.Count - 1]).Year;
-			int min_year = ((YearData)years [0]).Year;
+			int min_year = startyear;
+			int max_year = endyear;
 
 			if (year < min_year || year > max_year) {
-				Console.WriteLine("TimeAdaptor.IndexFromDate year out of range[{1},{2}]: {0}", year, min_year, max_year);
+				Log.DebugFormat ("TimeAdaptor.IndexFromDate year out of range[{1},{2}]: {0}", year, min_year, max_year);
 				return 0;
 			}
 
-			int index = date.Month - 1;
-
-			for (int i = 0 ; i < years.Count; i++)
-				if (year > ((YearData)years[i]).Year)
-					index += 12;
-
-			return index;	
+			return (year - startyear) * 12 + date.Month - 1 ;
 		}
 
 		private int IndexFromDateDescending(DateTime date)
 		{
 			int year = date.Year;
-			int max_year = ((YearData)years [0]).Year;
-			int min_year = ((YearData)years [years.Count - 1]).Year;
+			int min_year = startyear;
+			int max_year = endyear;
 		
 			if (year < min_year || year > max_year) {
-				Console.WriteLine("TimeAdaptor.IndexFromPhoto year out of range[{1},{2}]: {0}", year, min_year, max_year);
+				Log.DebugFormat ("TimeAdaptor.IndexFromPhoto year out of range[{1},{2}]: {0}", year, min_year, max_year);
 				return 0;
 			}
 
-			int index = 12 - date.Month;
-
-			for (int i = 0; i < years.Count; i++)
-				if (year < ((YearData)years[i]).Year)
-					index += 12;
-
-			return index;
+			return 12 * (endyear - year) + 12 - date.Month;
 		}
 
 		public override FSpot.IBrowsableItem PhotoFromIndex (int item)
 	       	{
 			DateTime start = DateFromIndex (item);
-			return query.Items [LookupItem (start)];
+			return query [query.LookupItem (start)];
 		
 		}
 
 		public override event ChangedHandler Changed;
 		
+		uint timer;
 		protected override void Reload () 
 		{
-			years.Clear ();
-
-			Photo [] photos = query.Store.Query ((Tag [])null, null, null, null, null);
-
-			Array.Sort (query.Photos);
-			Array.Sort (photos);
-
-			if (!order_ascending) {				
-				Array.Reverse (query.Photos);
-				Array.Reverse (photos);
-			}
-
-			if (photos.Length > 0) {
-				YearData data = new YearData ();
-				data.Year = 0;
-
-				foreach (Photo photo in photos) {
-					int current = photo.Time.Year;
-					if (current != data.Year) {
-						
-						data.Year = current;
-						data.Months = new int [12];
-						years.Add (data);
-						//Console.WriteLine ("Found Year {0}", current);
-					}
-					if (order_ascending)
-						data.Months [photo.Time.Month - 1] += 1;
-					else
-						data.Months [12 - photo.Time.Month] += 1;
-				}
-			} else {
-				YearData data = new YearData ();
-				data.Year = DateTime.Now.Year;
-				data.Months = new int [12];
-				years.Add (data);
-			}
-
- 			if (Changed != null)
-				Changed (this);
+			timer = Log.DebugTimerStart ();
+			Thread reload = new Thread (new ThreadStart (DoReload));
+			reload.IsBackground = true;
+			reload.Priority = ThreadPriority.Lowest;
+			reload.Start ();
 		}
 
+		int startyear = Int32.MaxValue, endyear = Int32.MinValue;
+		void DoReload ()
+		{
+			Thread.Sleep (200);
+			years = query.Store.PhotosPerMonth ();
+			foreach (int year in years.Keys) {
+				startyear = Math.Min (year,startyear);
+				endyear = Math.Max (year,endyear);
+			}
+ 			if (Changed != null)
+				Gtk.Application.Invoke(delegate {
+					if (Changed != null)
+						Changed (this);
+				});
+			Log.DebugTimerPrint (timer, "TimeAdaptor REAL Reload took {0}");
+		}
 
 		public TimeAdaptor (PhotoQuery query, bool order_ascending) 
 			: base (query, order_ascending)
