@@ -7,20 +7,21 @@ using FSpot.Widgets;
 using FSpot.UI.Dialog;
 
 namespace FSpot {
-	public class TimeChangedEventArgs : DbItemEventArgs {
-		TimeSpan span;
-
-		public TimeChangedEventArgs (DbItem [] items, TimeSpan span) : base (items)
-		{
-			this.span = span;
-		}
-
-		public TimeSpan TimeSpan {
-			get {
-				return span;
-			}
-		}
-	}
+//	public class TimeChangedEventArgs : PhotoEventArgs {
+//		TimeSpan span;
+//
+//		public TimeChangedEventArgs (Photo [] items, TimeSpan span)
+//			: base (items, true, false)
+//		{
+//			this.span = span;
+//		}
+//
+//		public TimeSpan TimeSpan {
+//			get {
+//				return span;
+//			}
+//		}
+//	}
 
 	public class TimeDialog : GladeDialog 
 	{
@@ -138,7 +139,7 @@ namespace FSpot {
 				old_label.Text = item.Time.ToLocalTime ().ToString ();
 				
 				int i = collection.Count > 0 ? Item.Index + 1: 0;
-				// This indicates the current photo is photo {0} of {1} out of photos
+				// Note for translators: This indicates the current photo is photo {0} of {1} out of photos
 				count_label.Text = System.String.Format (Catalog.GetString ("{0} of {1}"), i, collection.Count);
 
 				DateTime actual = item.Time;
@@ -168,7 +169,7 @@ namespace FSpot {
 				System.Console.WriteLine ("XXXXX old: {0} new: {1} span: {2}", time, p.Time, span);
 			}
 			
-			db.Photos.Commit (photos, new TimeChangedEventArgs (photos, span));
+			db.Photos.Commit (photos);
 		}
 
 		private void SpaceByInterval ()
@@ -192,7 +193,7 @@ namespace FSpot {
 				accum += span;
 			}
 			
-			db.Photos.Commit (photos, new TimeChangedEventArgs (photos, span));
+			db.Photos.Commit (photos);
 		}
 
 		void HandleSpinChanged (object sender, EventArgs args)
