@@ -246,7 +246,109 @@ namespace FSpot.Database {
 				Execute ("UPDATE photos SET rating = 0 WHERE rating IS NULL");
 			});
 
-			// Update to version 14.0
+			// Update to version 15.0
+			AddUpdate (new Version (15,0), delegate () {
+				string tmp_photo_tags = MoveTableToTemp ("photo_tags");
+				Execute (
+					"CREATE TABLE photo_tags (        " +
+					"	photo_id      INTEGER,    " +
+					"       tag_id        INTEGER,    " +
+					"       UNIQUE (photo_id, tag_id) " +
+					")");
+				Execute (String.Format (
+					"INSERT OR IGNORE INTO photo_tags (photo_id, tag_id) " +
+					"SELECT photo_id, tag_id FROM {0}", tmp_photo_tags));
+				string tmp_photo_versions = MoveTableToTemp ("photo_versions");
+				Execute (
+					"CREATE TABLE photo_versions (		"+
+					"	photo_id	INTEGER,	" +
+					"	version_id	INTEGER,	" +
+					"	name		STRING,		" +
+					"	uri		STRING NOT NULL," +
+					"	protected	BOOLEAN, 	" +
+					"	UNIQUE (photo_id, version_id)	" +
+					")");
+				Execute (String.Format (
+					"INSERT OR IGNORE INTO photo_versions 		" +
+					"(photo_id, version_id, name, uri, protected)	" +
+					"SELECT photo_id, version_id, name, uri, protected FROM {0}", tmp_photo_versions));
+			});
+
+			// Update to version 16.0
+			 AddUpdate (new Version (16,0), delegate () {
+				 string temp_table = MoveTableToTemp ("photos");
+  
+				 Execute ("CREATE TABLE photos ( " +
+					  "	id                 INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,   " +
+					  "	time               INTEGER NOT NULL,	   	   " +
+					  "	uri		   STRING NOT NULL,		   " +
+					  "	description        TEXT NOT NULL,	           " +
+					  "	roll_id            INTEGER NOT NULL,		   " +
+					  "	default_version_id INTEGER NOT NULL,		   " +
+					  "	rating		   INTEGER NULL,		   " +
+					  "	md5_sum		   TEXT NULL  			   " +
+					  ")"
+				 );
+  
+				 Execute (string.Format ("INSERT INTO photos (id, time, uri, description, roll_id, " + 
+							 "default_version_id, rating, md5_sum) " + 
+							 "SELECT id, time, uri, description, roll_id, " +
+							 "       default_version_id, rating, '' " +
+							 "FROM   {0} ", 
+							 temp_table
+							)
+				 );
+
+
+				 string temp_versions_table = MoveTableToTemp ("photo_versions");
+
+				 Console.WriteLine("{0} - {1}", temp_table, temp_versions_table);
+
+				 Execute ("CREATE TABLE photo_versions (    	" +
+					  "      photo_id        INTEGER,  	" +
+					  "      version_id      INTEGER,  	" +
+					  "      name            STRING,    	" +
+					  "	uri		STRING NOT NULL," +
+					  "	md5_sum		STRING NOT NULL," +
+					  "	protected	BOOLEAN		" +
+					  ")");
+
+				 Execute (string.Format ("INSERT INTO photo_versions (photo_id, version_id, name, uri, md5_sum, protected) " + 
+							 "SELECT photo_id, version_id, name, uri, '', protected " +
+							 "FROM   {0} ", 
+							 temp_versions_table
+							)
+				 );
+
+				 // This is kind of hacky but should be a lot faster on
+				 // large photo databases
+				 Execute (string.Format ("INSERT INTO jobs (job_type, job_options, run_at, job_priority) " +
+							 "SELECT '{0}', id, {1}, {2} " +
+							 "FROM   photos ",
+							 typeof(Jobs.CalculateHashJob).ToString (),
+							 FSpot.Utils.DbUtils.UnixTimeFromDateTime (DateTime.Now),
+							 0
+							)
+				 );
+			 }, true);
+
+			// Update to version 16.1
+			 AddUpdate (new Version (16,1), delegate () {
+				 Execute ("CREATE INDEX idx_photo_versions_id ON photo_versions(photo_id)");
+			 }, false);
+
+			// Update to version 16.2
+			 AddUpdate (new Version (16,2), delegate () {
+				 Execute ("CREATE INDEX idx_photos_roll_id ON photos(roll_id)");
+			 }, false);
+
+			// Update to version 16.3
+			AddUpdate (new Version (16,3), delegate () {
+				Execute (String.Format ("DELETE FROM jobs WHERE job_type = '{0}'", typeof(Jobs.CalculateHashJob).ToString ()));
+			}, false);
+
+
+			 // Update to version 17.0
 			//AddUpdate (new Version (14,0),delegate () {
 			//	do update here
 			//});
