@@ -194,7 +194,7 @@ namespace FSpot.Widgets
 		private int click_count;
 
 		// Public events.
-		public delegate void DoubleClickedHandler (IconView view, int clicked_item);
+		public delegate void DoubleClickedHandler (Widget widget, BrowsableEventArgs args);
 		public event DoubleClickedHandler DoubleClicked;
 
 		public delegate void ZoomChangedHandler (object sender, System.EventArgs args);
@@ -258,7 +258,8 @@ namespace FSpot.Widgets
 		private void HandleItemsChanged (FSpot.IBrowsableCollection sender, BrowsableEventArgs args)
 		{
 			foreach (int item in args.Items) {
-				UpdateThumbnail (item);
+				if (args.Changes.DataChanged)
+					UpdateThumbnail (item);
 				InvalidateCell (item);
 			}
 		}
@@ -325,11 +326,11 @@ namespace FSpot.Widgets
 
 			}
 
-			public void MarkChanged (int item)
+			public void MarkChanged (int item, IBrowsableItemChanges changes)
 			{
 				// Forward the change event up to our parent
 				// we'll fire the event when the parent calls us back
-				parent.MarkChanged ((int) selected_cells [item]);
+				parent.MarkChanged ((int) selected_cells [item], changes);
 			}
 
 			private void HandleParentItemsChanged (IBrowsableCollection collection, BrowsableEventArgs args)
@@ -351,7 +352,8 @@ namespace FSpot.Widgets
 				if (local_ids.Count == 0)
 					return;
 
-				ItemsChanged (this, new BrowsableEventArgs ((int [])local_ids.ToArray (typeof (int))));
+				int [] items = (int [])local_ids.ToArray (typeof (int));
+				ItemsChanged (this, new BrowsableEventArgs (items, args.Changes));
 			}
 
 			public int [] Ids {
@@ -867,6 +869,11 @@ namespace FSpot.Widgets
 						draw.Width, draw.Height);
 
 				if (region.Intersect (area, out draw)) {
+					//FIXME
+					if (FSpot.ColorManagement.IsEnabled) {
+						temp_thumbnail = temp_thumbnail.Copy();
+						FSpot.ColorManagement.ApplyScreenProfile (temp_thumbnail);
+					}
 					temp_thumbnail.RenderToDrawable (BinWindow, Style.WhiteGC,
 							draw.X - region.X,
 							draw.Y - region.Y,
@@ -986,6 +993,8 @@ namespace FSpot.Widgets
 									tag_bounds.Height,
 									InterpType.Bilinear);
 						}
+						
+						FSpot.ColorManagement.ApplyScreenProfile (scaled_icon);
 
 						scaled_icon.RenderToDrawable (BinWindow, Style.WhiteGC,
 								region.X - tag_bounds.X,
@@ -1030,7 +1039,7 @@ namespace FSpot.Widgets
 			i ++) {
 				int cell_x = start_cell_x;
 
-				//Console.WriteLine ("Drawing row {0}", start_cell_row + i);
+				//Log.DebugFormat ("Drawing row {0}", start_cell_row + i);
 				for (int j = 0; j < num_cols && cell_num + j < collection.Count; j ++) {
 					DrawCell (cell_num + j, area);
 					cell_x += cell_width;
@@ -1079,7 +1088,7 @@ namespace FSpot.Widgets
 
 			Gdk.Region offscreen = new Gdk.Region ();
 			/*
-			System.Console.WriteLine ("step ({0}, {1}) allocation ({2},{3},{4},{5})",
+			Log.DebugFormat ("step ({0}, {1}) allocation ({2},{3},{4},{5})",
 					xstep, ystep, Hadjustment.Value, Vadjustment.Value,
 					Allocation.Width, Allocation.Height);
 			*/
@@ -1296,7 +1305,7 @@ namespace FSpot.Widgets
 			int width, height;
 			PixbufUtils.Fit (result, ThumbnailWidth, ThumbnailHeight, false, out width, out height);
 			if (result.Width > width && result.Height > height) {
-				//  System.Console.WriteLine ("scaling");
+				//  Log.Debug ("scaling");
 				Gdk.Pixbuf temp = PixbufUtils.ScaleDown (result, width, height);
 				result.Dispose ();
 				result = temp;
@@ -1422,12 +1431,11 @@ namespace FSpot.Widgets
 
 			switch (args.Event.Type) {
 			case EventType.TwoButtonPress:
-				if (args.Event.Button != 1
-					|| (args.Event.State &  (ModifierType.ControlMask
-				| ModifierType.ShiftMask)) != 0)
-				return;
+				if (args.Event.Button != 1 ||
+						(args.Event.State &  (ModifierType.ControlMask | ModifierType.ShiftMask)) != 0)
+					return;
 				if (DoubleClicked != null)
-					DoubleClicked (this, cell_num);
+					DoubleClicked (this, new BrowsableEventArgs (cell_num, null));
 				return;
 
 			case EventType.ButtonPress:
@@ -1542,9 +1550,8 @@ namespace FSpot.Widgets
 				ToggleCell (FocusCell);
 				break;
 			case Gdk.Key.Return:
-				if (DoubleClicked == null)
-					break;
-				DoubleClicked (this, FocusCell);
+				if (DoubleClicked != null)
+					DoubleClicked (this, new BrowsableEventArgs (FocusCell, null));
 				break;
 			default:
 				args.RetVal = false;
