@@ -15,14 +15,12 @@ using Gdk;
 using System;
 using Cairo;
 using FSpot;
+using FSpot.Utils;
 
 namespace FSpot.Widgets {
 	[Binding(Gdk.Key.Up, "Up")]
 	[Binding(Gdk.Key.Down, "Down")]
-	[Binding(Gdk.Key.Left, "TiltImage", 0.05)]
-	[Binding(Gdk.Key.Right, "TiltImage", -0.05)] 
 	[Binding(Gdk.Key.space, "Pan")]
-	[Binding(Gdk.Key.Q, "Vingette")]
 	[Binding(Gdk.Key.R, "RevealImage")]
 	[Binding(Gdk.Key.P, "PushImage")]
 	public class ImageDisplay : Gtk.EventBox {
@@ -96,35 +94,6 @@ namespace FSpot.Widgets {
 			return true;
 		}
 
-		public bool Vingette ()
-		{
-			SoftFocus f = effect as SoftFocus;
-			
-			if (f == null) {
-				f = new SoftFocus (current);
-				effect = f;
-			}
-
-			QueueDraw ();
-			return true;
-		}
-
-		public bool TiltImage (double radians)
-		{
-			Tilt t = effect as Tilt;
-
-			if (t == null) {
-				t = new Tilt (current);
-				effect = t;
-			}
-
-			t.Angle += radians;
-
-			QueueDraw ();
-
-			return true;
-		}
-
 		public bool Pan ()
 		{
 			Console.WriteLine ("space");
@@ -236,7 +205,7 @@ namespace FSpot.Widgets {
 			bool double_buffer = false;
 			base.OnExposeEvent (args);
 
-			Context ctx = CairoUtils.CreateContext (GdkWindow);
+			Context ctx = Gdk.CairoHelper.Create (GdkWindow);
 			//Surface glitz = CairoUtils.CreateGlitzSurface (GdkWindow);
 			//Context ctx = new Context (glitz);
 			if (double_buffer) {
