@@ -12,6 +12,7 @@ using Mono.Addins;
 using Mono.Addins.Setup;
 using FSpot.Utils;
 using FSpot.UI.Dialog;
+using FSpot.Extensions;
 
 namespace FSpot 
 {
@@ -60,7 +61,7 @@ public class Driver {
 		Program program = null;
 		ICore control = null;
 		List<string> uris = new List<string> ();
-		SetProcessName (Defines.PACKAGE);
+		Unix.SetProcessName (Defines.PACKAGE);
 
 		// Options and Option parsing
 		bool shutdown = false;
@@ -68,15 +69,11 @@ public class Driver {
 		bool slideshow = false;
 		string import_uri = null;
 		
-		// Debug GdkPixbuf critical warnings
-		//LogFunc logFunc = new GLib.LogFunc (GLib.Log.PrintTraceLogFunction);
-		//Log.SetLogHandler ("GdkPixbuf", GLib.LogLevelFlags.Critical, logFunc);
-
 		program = new Program (Defines.PACKAGE, 
 				       Defines.VERSION, 
 				       Modules.UI, args);		
 		
-		FSpot.Global.PhotoDirectory = (string) Preferences.Get(Preferences.STORAGE_PATH);
+		FSpot.Global.PhotoDirectory = Preferences.Get<string> (Preferences.STORAGE_PATH);
 
 		for (int i = 0; i < args.Length && !shutdown; i++) {
 			switch (args [i]) {
@@ -85,31 +82,31 @@ public class Driver {
 				return 0;
 			
 			case "-shutdown": case "--shutdown":
-				System.Console.WriteLine ("Shutting down existing F-Spot server...");
+				Log.Information ("Shutting down existing F-Spot server...");
 				shutdown = true;
 				break;
 
 			case "-b": case "-basedir": case "--basedir":
 				if (i+1 == args.Length || args[i+1].StartsWith ("-")) {
-					Console.WriteLine ("f-spot: -basedir option takes one argument");
+					Log.Error ("f-spot: -basedir option takes one argument");
 					return 1;
 				}
 				FSpot.Global.BaseDirectory = args [++i];
-				System.Console.WriteLine ("BaseDirectory is now {0}", FSpot.Global.BaseDirectory);
+				Log.InformationFormat ("BaseDirectory is now {0}", FSpot.Global.BaseDirectory);
 				break;
 
 			case "-p": case "-photodir": case "--photodir":
 				if (i+1 == args.Length || args[i+1].StartsWith ("-")) {
-					Console.WriteLine ("f-spot: -photodir option takes one argument");
+					Log.Error ("f-spot: -photodir option takes one argument");
 					return 1;
 				}
 				FSpot.Global.PhotoDirectory = System.IO.Path.GetFullPath (args [++i]);
-				System.Console.WriteLine ("PhotoDirectory is now {0}", FSpot.Global.PhotoDirectory);
+				Log.InformationFormat ("PhotoDirectory is now {0}", FSpot.Global.PhotoDirectory);
 				break;
 
 			case "-i": case "-import": case "--import":
 				if (i+1 == args.Length) {
-					Console.WriteLine ("f-spot: -import option takes one argument");
+					Log.Error ("f-spot: -import option takes one argument");
 					return 1;
 				}
 				import_uri = args [++i];
@@ -121,14 +118,14 @@ public class Driver {
 
 			case "-v": case "-view": case "--view":
 				if (i+1 == args.Length || args[i+1].StartsWith ("-")) {
-					Console.WriteLine ("f-spot: -view option takes (at least) one argument");
+					Log.Error ("f-spot: -view option takes (at least) one argument");
 					return 1;
 				}
 				view = true;
 				while (!(i+1 == args.Length) && !args[i+1].StartsWith ("-"))
 					uris.Add (args [++i]);
 //				if (!System.IO.Directory.Exists (args[i+1]) && !System.IO.File.Exists (args[i+1])) {
-//					Console.WriteLine ("f-spot: -view argument must be an existing file or directory");
+//					Log.Error ("f-spot: -view argument must be an existing file or directory");
 //					return 1;
 //				}
 				break;
@@ -143,11 +140,22 @@ public class Driver {
 			
 			case "--debug":
 				Log.Debugging = true;
-				break;
-			case "--trace": case "--profile": case "--uninstalled": case "--gdb":
-				break;
+				// Debug GdkPixbuf critical warnings
+				GLib.LogFunc logFunc = new GLib.LogFunc (GLib.Log.PrintTraceLogFunction);
+				GLib.Log.SetLogHandler ("GdkPixbuf", GLib.LogLevelFlags.Critical, logFunc);
+		
+				// Debug Gtk critical warnings
+				GLib.Log.SetLogHandler ("Gtk", GLib.LogLevelFlags.Critical, logFunc);
 
+				break;
+			case "--uninstalled": case "--gdb":
+				break;
 			default:
+				if (args [i].StartsWith ("--profile"))
+					break;
+				if (args [i].StartsWith ("--trace"))
+					break;
+				Log.DebugFormat ("Unparsed argument >>{0}<<", args [i]);
 				Help ();
 				return 1;
 			}
@@ -157,7 +165,7 @@ public class Driver {
 		if ( (import_uri != null && (view || shutdown || slideshow)) || 
 		     (view && (shutdown || slideshow)) ||
 		     (shutdown && slideshow) ) {
-			Console.WriteLine ("Can't mix -import, -view, -shutdown or -slideshow");
+			Log.Error ("Can't mix -import, -view, -shutdown or -slideshow");
 			return 1;
 		}
 
@@ -167,7 +175,7 @@ public class Driver {
 			Core core = new Core ();
 			core.ShowSlides (null);
 			program.Run ();
-			System.Console.WriteLine ("done");
+			Log.Debug ("done");
 			return 0;
 		}
 		
@@ -187,11 +195,13 @@ public class Driver {
 			string maj_version = String.Join (".", Defines.VERSION.Split ('.'), 0, 3);
 			foreach (AddinRepository repo in setupService.Repositories.GetRepositories ())
 				if (repo.Url.StartsWith ("http://addins.f-spot.org/") && !repo.Url.StartsWith ("http://addins.f-spot.org/" + maj_version)) {
-					Console.WriteLine ("Unregistering {0}", repo.Url);
+					Log.InformationFormat ("Unregistering {0}", repo.Url);
 					setupService.Repositories.RemoveRepository (repo.Url);
 				}
 			setupService.Repositories.RegisterRepository (null, "http://addins.f-spot.org/" + maj_version, false);
 			Log.DebugTimerPrint (ma_timer, "Mono.Addins Initialization took {0}");
+			
+		
 
 			bool create = true;
 			int retry_count = 0;
@@ -211,14 +221,14 @@ public class Driver {
 						create = false;
 						Gnome.Vfs.Vfs.Initialize ();
 
-						if (File.Exists (Preferences.Get (Preferences.GTK_RC) as string)) {
+						if (File.Exists (Preferences.Get<string> (Preferences.GTK_RC))) {
 #if GTK_2_12_2
 							if (!File.Exists (Path.Combine (Global.BaseDirectory, "gtkrc")))
 								(File.Create (Path.Combine (Global.BaseDirectory, "gtkrc"))).Dispose ();
 							Gtk.Rc.AddDefaultFile (Path.Combine (Global.BaseDirectory, "gtkrc"));
 							Global.DefaultRcFiles = Gtk.Rc.DefaultFiles;
 #endif
-							Gtk.Rc.AddDefaultFile (Preferences.Get (Preferences.GTK_RC) as string);
+							Gtk.Rc.AddDefaultFile (Preferences.Get<string> (Preferences.GTK_RC));
 						}
 						
 						Catalog.Init ("f-spot", Defines.LOCALE_DIR);
@@ -233,8 +243,7 @@ public class Driver {
 
 						core = new Core (view);
 						core.RegisterServer ();
-						
-						
+
 						empty = view || Core.Database.Empty;
 						control = core;
 					}
@@ -251,9 +260,9 @@ public class Driver {
 					}
 				}
 				if (control == null) {
-					System.Console.WriteLine ("Can't get a connection to the dbus. Trying again...");
+					Log.Warning ("Can't get a connection to the dbus. Trying again...");
 					if (++ retry_count > 5) {
-						System.Console.WriteLine ("Sorry, couldn't start F-Spot");
+						Log.Error ("Sorry, couldn't start F-Spot");
 						return 1;
 					}
 				}
@@ -291,14 +300,48 @@ public class Driver {
 			if (import_uri != null || !view) {
 				control.Organize ();
 				Gdk.Global.NotifyStartupComplete ();
+				foreach (ServiceNode service in AddinManager.GetExtensionNodes ("/FSpot/Services")) {
+					service.Initialize ();
+					service.Start ();
+				}
 			}			
 
+#if GSD_2_24
+			Log.Information ("Hack for gnome-settings-daemon engaged");
+			int max_age, max_size;
+			if (Preferences.TryGet<int> (Preferences.GSD_THUMBS_MAX_AGE, out max_age)) {
+				if (max_age < 0)
+					Log.Debug ("maximum_age check already disabled, good");
+				else if (max_age == 0)
+					Log.Warning ("maximum_age is 0 (tin-hat mode), not overriding");
+				else if (max_age < 180) {
+					Log.Debug ("Setting maximum_age to a saner value");
+					Preferences.Set (Preferences.GSD_THUMBS_MAX_AGE, 180);
+				}
+			}
+
+			if (Preferences.TryGet<int> (Preferences.GSD_THUMBS_MAX_SIZE, out max_size)) {
+				int count = Core.Database.Photos.Count ("photos");
+				// average thumbs are taking 70K, so this will push the threshold
+				//if f-spot takes more than 70% of the thumbs space
+				int size = count / 10;
+				if (max_size < 0)
+					Log.Debug ("maximum_size check already disabled, good");
+				else if (max_size == 0)
+					Log.Warning ("maximum_size is 0 (tin-hat mode), not overriding");
+				else if (max_size < size) {
+					Log.DebugFormat ("Setting maximum_size to a saner value ({0}MB), according to your db size", size);
+					Preferences.Set (Preferences.GSD_THUMBS_MAX_SIZE, size);
+				}
+			}
+
+#endif
 			if (program != null)
 				program.Run ();
 			
-			System.Console.WriteLine ("exiting");
+			Log.Information ("exiting");
 		} catch (System.Exception e) {
-			Console.Error.WriteLine(e);
+			Log.Exception (e);
 			ExceptionDialog dlg = new ExceptionDialog(e);
 			dlg.Run();
 			dlg.Destroy();
@@ -306,25 +349,5 @@ public class Driver {
 		}
 		return 0;
 	}
-
-	[DllImport("libc")]
-	private static extern int prctl(int option, string name, ulong arg3,
-					ulong arg4, ulong arg5);
-	
-	public static void SetProcessName(string name)
-	{
-		try {
-			if(prctl(15 /* PR_SET_NAME */, name,
-				 0, 0, 0) != 0) {
-				Console.WriteLine ("Error setting process name: " +
-						   Mono.Unix.Native.Stdlib.GetLastError());
-			}
-		} catch (DllNotFoundException) {
-			/* noop */
-		} catch (EntryPointNotFoundException) {
-		    	/* noop */
-		}
-	}
 }
-
 }
